import 'package:flutter/widgets.dart';
import 'extendedAlignments.dart';

extension ExtendedColumn on Column {
  /// Returns a Column with any number of the properties changed.
  ///
  /// This is used for the library to change the Column properties. You can use
  /// this to change properties after creating a Column.
  /// Use it like so:
  /// ```dart
  /// Column(...).copyWith(mainAxisSize: MainAxisSize.max);
  /// ```
  /// This can be used to create a Column and change properties afterwards.
  Column copyWith({
    List<Widget> children,
    TextDirection textDirection,
    MainAxisAlignment mainAxisAlignment,
    CrossAxisAlignment crossAxisAlignment,
    MainAxisSize mainAxisSize,
    TextBaseline textBaseline,
    VerticalDirection verticalDirection,
  }) {
    return Column(
      children: children ?? this.children,
      textDirection: textDirection ?? this.textDirection,
      mainAxisAlignment: mainAxisAlignment ?? this.mainAxisAlignment,
      crossAxisAlignment: crossAxisAlignment ?? this.crossAxisAlignment,
      mainAxisSize: mainAxisSize ?? this.mainAxisSize,
      textBaseline: textBaseline ?? this.textBaseline,
      verticalDirection: verticalDirection ?? this.verticalDirection,
    );
  }

  /// Allows adding Widgets to a Column
  ///
  /// When a Widget is being added to a Column, if that Widget is a Column, then
  /// its children are appended to the children of the first widget, and if that
  /// Widget is anything else, that widget is appended to the children of the
  /// Column.
  /// The Following will result in one Column:
  /// ```dart
  /// Column(...) + Container();
  /// Column(...) + Column(...);
  /// ```
  Column operator +(Widget combine) {
    if (combine is Column) {
      return this.copyWith(children: [...this.children, ...combine.children]);
    } else {
      return this.copyWith(children: [...this.children, combine]);
    }
  }

  /// Turns a Column into A Row
  ///
  /// Most of the properties of the Column will be preserved where possible.
  ///
  /// Use it like so:
  /// ```dart
  /// Column(...).transpose();
  /// ```
  Row transpose() {
    return Row(
      children: this.children,
      verticalDirection: this.verticalDirection,
      textBaseline: this.textBaseline,
      mainAxisSize: this.mainAxisSize,
      textDirection: this.textDirection,
      crossAxisAlignment: this.crossAxisAlignment,
      mainAxisAlignment: this.mainAxisAlignment,
    );
  }

  /// Turns a Column into A Wrap that has a vertical direction
  ///
  /// Most of the properties of the Column will be preserved where possible.
  ///
  /// Use it like so:
  /// ```dart
  /// Column(...).wrap();
  /// ```
  Wrap wrap({
    double spacing,
    double runSpacing,
    WrapAlignment runAlignment,
  }) {
    return Wrap(
      children: this.children,
      textDirection: this.textDirection,
      verticalDirection: this.verticalDirection,
      direction: Axis.vertical,
      spacing: spacing,
      runSpacing: runSpacing,
      runAlignment: runAlignment,
      alignment: this.mainAxisAlignment.wrapAlignment(),
      crossAxisAlignment: this.crossAxisAlignment.wrapCrossAlignment(),
    );
  }
}
