import 'package:flutter/widgets.dart';
import 'extendedAlignments.dart';

extension ExtendedRow on Row {
  /// Returns a Row with any number of the properties changed.
  ///
  /// This is used for the library to change the Row properties. You can use
  /// this to change properties after creating a Row.
  /// Use it like so:
  /// ```dart
  /// Row(...).copyWith(mainAxisSize: MainAxisSize.max);
  /// ```
  /// This can be used to create a Row and change properties afterwards.
  Row copyWith({
    List<Widget> children,
    TextDirection textDirection,
    MainAxisAlignment mainAxisAlignment,
    CrossAxisAlignment crossAxisAlignment,
    MainAxisSize mainAxisSize,
    TextBaseline textBaseline,
    VerticalDirection verticalDirection,
  }) {
    return Row(
      children: children ?? this.children,
      textDirection: textDirection ?? this.textDirection,
      mainAxisAlignment: mainAxisAlignment ?? this.mainAxisAlignment,
      crossAxisAlignment: crossAxisAlignment ?? this.crossAxisAlignment,
      mainAxisSize: mainAxisSize ?? this.mainAxisSize,
      textBaseline: textBaseline ?? this.textBaseline,
      verticalDirection: verticalDirection ?? this.verticalDirection,
    );
  }

  /// Allows adding Widgets to a Row
  ///
  /// When a Widget is being added to a Row, if that Widget is a Row, then
  /// its children are appended to the children of the first widget, and if that
  /// Widget is anything else, that widget is appended to the children of the
  /// Row.
  /// The Following will result in one Row:
  /// ```dart
  /// Row(...) | Container();
  /// Row(...) | Row(...);
  /// ```
  Row operator |(Widget combine) {
    if (combine is Row) {
      return this.copyWith(children: [...this.children, ...combine.children]);
    } else {
      return this.copyWith(children: [...this.children, combine]);
    }
  }

  /// Turns a Row into A Column
  ///
  /// Most of the properties of the Row will be preserved where possible.
  ///
  /// Use it like so:
  /// ```dart
  /// Row(...).transpose();
  /// ```
  Column transpose() {
    return Column(
      children: this.children,
      verticalDirection: this.verticalDirection,
      textBaseline: this.textBaseline,
      mainAxisSize: this.mainAxisSize,
      textDirection: this.textDirection,
      crossAxisAlignment: this.crossAxisAlignment,
      mainAxisAlignment: this.mainAxisAlignment,
    );
  }

  /// Turns a Row into A Wrap that has a vertical direction
  ///
  /// Most of the properties of the Row will be preserved where possible.
  ///
  /// Use it like so:
  /// ```dart
  /// Row(...).wrap();
  /// ```
  Wrap wrap({
    double spacing,
    double runSpacing,
    WrapAlignment runAlignment,
  }) {
    return Wrap(
      children: this.children,
      textDirection: this.textDirection,
      verticalDirection: this.verticalDirection,
      direction: Axis.horizontal,
      spacing: spacing,
      runSpacing: runSpacing,
      runAlignment: runAlignment,
      alignment: this.mainAxisAlignment.wrapAlignment(),
      crossAxisAlignment: this.crossAxisAlignment.wrapCrossAlignment(),
    );
  }
}
