import 'package:flutter/widgets.dart';

extension ExtendedStack on Stack {
  /// Returns a Stack with any number of the properties changed.
  ///
  /// This is used for the library to change the Stack properties. You can use
  /// this to change properties after creating a Stack.
  /// Use it like so:
  /// ```dart
  /// Stack(...).copyWith(fit: StackFit.expand);
  /// ```
  /// This can be used to create a Stack and change properties afterwards.
  Stack copyWith({
    List<Widget> children,
    TextDirection textDirection,
    AlignmentGeometry alignment,
    Overflow overflow,
    StackFit fit,
  }) {
    return Stack(
      children: children ?? this.children,
      textDirection: textDirection ?? this.textDirection,
      alignment: alignment ?? this.alignment,
      overflow: overflow ?? this.overflow,
      fit: fit ?? this.fit,
    );
  }

  /// Allows adding Widgets to a Stack
  ///
  /// When a Widget is being added to a Stack, if that Widget is a Stack, then
  /// its children are appended to the children of the first widget, and if that
  /// Widget is anything else, that widget is appended to the children of the
  /// Stack.
  /// The Following will result in one Stack:
  /// ```dart
  /// Stack(...) & Container();
  /// Stack(...) & Stack(...);
  /// ```
  Stack operator &(Widget combine) {
    if (combine is Stack) {
      return this.copyWith(children: [...this.children, ...combine.children]);
    } else {
      return this.copyWith(children: [...this.children, combine]);
    }
  }
}
