import 'package:flutter/widgets.dart';

extension ExtendedText on Text {
  /// Returns a Text with any number of the properties changed.
  ///
  /// This is used for the library to change the Text properties. You can use
  /// this to change properties after creating a Text.
  /// Use it like so:
  /// ```dart
  /// Text(...).copyWith(softWrap: true);
  /// ```
  /// This can be used to create a Text and change properties afterwards.
  Text copyWith({
    String data,
    int maxLines,
    Locale locale,
    TextAlign textAlign,
    TextStyle style,
    TextOverflow overflow,
    String semanticsLabel,
    bool softWrap,
    StrutStyle strutStyle,
    TextDirection textDirection,
    double textScaleFactor,
    TextWidthBasis textWidthBasis,
  }) {
    return Text(
      data ?? this.data,
      maxLines: maxLines ?? this.maxLines,
      locale: locale ?? this.locale,
      textAlign: textAlign ?? this.textAlign,
      style: style ?? this.style,
      overflow: overflow ?? this.overflow,
      semanticsLabel: semanticsLabel ?? this.semanticsLabel,
      softWrap: softWrap ?? this.softWrap,
      strutStyle: strutStyle ?? this.strutStyle,
      textDirection: textDirection ?? this.textDirection,
      textScaleFactor: textScaleFactor ?? this.textScaleFactor,
      textWidthBasis: textWidthBasis ?? this.textWidthBasis,
    );
  }

  /// Changes the size of the Text widget
  ///
  /// To use it:
  /// ```dart
  /// Text('Hello World').size(10.0);
  /// ```
  Text size(double fontSize) {
    return this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(fontSize: fontSize));
  }

  /// Changes the color of the Text widget
  ///
  /// To use it:
  /// ```dart
  /// Text('Hello World').color(Colors.red);
  /// ```
  Text color(Color color) {
    return this
        .copyWith(style: (this.style ?? TextStyle()).copyWith(color: color));
  }

  /// Changes the font weight of the Text widget
  ///
  /// There is a property to change the font weight but by default the font
  /// weight is bold.
  /// To use it:
  /// ```dart
  /// Text('Hello World').bold();
  /// ```
  Text bold({FontWeight fontWeight = FontWeight.bold}) {
    return this.copyWith(
        style: (this.style ?? TextStyle()).copyWith(fontWeight: fontWeight));
  }

  /// Changes the italics of the Text widget
  ///
  /// To use it:
  /// ```dart
  /// Text('Hello World').italic();
  /// ```
  Text italic() {
    return this.copyWith(
        style:
            (this.style ?? TextStyle()).copyWith(fontStyle: FontStyle.italic));
  }

  /// Adds and underline to the text
  ///
  /// To use it:
  /// ```dart
  /// Text('Hello World').underline();
  /// ```
  Text underline() {
    return this.copyWith(
        style: (this.style ?? TextStyle())
            .copyWith(decoration: TextDecoration.underline));
  }

  /// Adds and overline to the text
  ///
  /// To use it:
  /// ```dart
  /// Text('Hello World').overline();
  /// ```
  Text overline() {
    return this.copyWith(
        style: (this.style ?? TextStyle())
            .copyWith(decoration: TextDecoration.overline));
  }

  /// Adds and a strike through to the text
  ///
  /// To use it:
  /// ```dart
  /// Text('Hello World').lineThrough();
  /// ```
  Text lineThrough() {
    return this.copyWith(
        style: (this.style ?? TextStyle())
            .copyWith(decoration: TextDecoration.lineThrough));
  }
}
