import 'package:flutter/widgets.dart';
import 'extendedColumn.dart';
import 'extendedRow.dart';
import 'extendedStack.dart';

extension ExtendedWidget on Widget {
  /// A convenience widget that will allow an empty placeholder widget
  ///
  /// To use it:
  /// ```dart
  /// true?Container:nothing()
  /// ```
  SizedBox nothing() {
    return const SizedBox(
      height: 0.0,
      width: 0.0,
    );
  }

  /// Adds padding to all sides of a Widget
  ///
  /// Allows adding padding in a concise manner. It effectively wraps the
  /// widget in the Padding Widget.
  /// To use it:
  /// ```dart
  /// Text('Hello World').paddingAll(10.0)
  /// ```
  Padding paddingAll(double value) {
    return Padding(
      padding: EdgeInsets.all(value),
      child: nothing(),
    );
  }

  /// Adds padding to vertical or horizontal sides of a Widget
  ///
  /// Allows adding padding in a concise manner. It effectively wraps the
  /// widget in the Padding Widget.
  /// To use it:
  /// ```dart
  /// Text('Hello World').paddingSymmetric(horizontal:10.0)
  /// ```
  Padding paddingSymmetric({double horizontal = 0.0, double vertical = 0.0}) {
    return Padding(
      padding: EdgeInsets.symmetric(horizontal: horizontal, vertical: vertical),
      child: this,
    );
  }

  /// Adds padding to any side of a Widget
  ///
  /// Allows adding padding in a concise manner. It effectively wraps the
  /// widget in the Padding Widget.
  /// To use it:
  /// ```dart
  /// Text('Hello World').paddingLRTB(left:10.0)
  /// ```
  Padding paddingLRTB(
      {double left = 0.0,
      double right = 0.0,
      double top = 0.0,
      double bottom = 0.0}) {
    return Padding(
      padding: EdgeInsets.fromLTRB(left, top, right, bottom),
      child: this,
    );
  }

  /// Centers the widget
  ///
  /// It effectively wraps the widget in the Center Widget.
  /// To use it:
  /// ```dart
  /// Text('Hello World').center()
  /// ```
  Center center({double heightFactor, double widthFactor}) {
    return Center(
      child: this,
      heightFactor: heightFactor,
      widthFactor: widthFactor,
    );
  }

  /// Wraps the widget with expanded
  ///
  /// THe flex factor is an optional parameter.
  /// To use it:
  /// ```dart
  /// Text('Hello World').expanded()
  /// ```
  Expanded expanded({int flex = 1}) {
    return Expanded(
      child: this,
      flex: flex,
    );
  }

  /// Wraps the widget with flexible
  ///
  /// THe flex factor and FlexFit is an optional parameter.
  /// To use it:
  /// ```dart
  /// Text('Hello World').flexible()
  /// ```
  Flexible flexible({int flex = 1, FlexFit fit = FlexFit.loose}) {
    return Flexible(
      child: this,
      flex: flex,
      fit: fit,
    );
  }

  /// Wraps the widget with Align Widget
  ///
  /// Requires an AlignmentGeometry and has optionals parameters for height
  /// factor and width factor.
  /// To use it:
  /// ```dart
  /// Text('Hello World').align(alignment : Alignment.topCenter)
  /// ```
  Align align(AlignmentGeometry alignment,
      {double heightFactor, double widthFactor}) {
    return Align(
      child: this,
      alignment: alignment,
      widthFactor: widthFactor,
      heightFactor: heightFactor,
    );
  }

  /// Wraps the widget with the Baseline Widget
  ///
  /// Requires an AlignmentGeometry and has optionals parameters for height
  /// factor and width factor.
  /// To use it:
  /// ```dart
  /// Text('Hello World').baseline(baselineType : TextBaseline.ideographic)
  /// ```
  Baseline baseline(
      {double baseline = 0.0,
      TextBaseline baselineType = TextBaseline.alphabetic}) {
    return Baseline(
      child: this,
      baseline: baseline,
      baselineType: baselineType,
    );
  }

  /// Wraps the widget with the Positioned Widget
  ///
  /// Allows use to position a widget in a stack.
  /// To use it:
  /// ```dart
  /// Text('Hello World').positioned(left: 5,0)
  /// ```
  Positioned positioned({
    double height = 0.0,
    double width = 0.0,
    double left = 0.0,
    double top = 0.0,
    double bottom = 0.0,
    double right = 0.0,
  }) {
    return Positioned(
      child: this,
      height: height,
      width: width,
      left: left,
      top: top,
      bottom: bottom,
      right: right,
    );
  }

  /// Wraps the widget with a Expanded ConstrainedBox
  ///
  /// This is a shorthand for ConstrainedBox that has the constraint
  /// BoxConstraints.expand
  /// To use it:
  /// ```dart
  /// Text('Hello World').expandedBox(width: 100.0, height: 100.0)
  /// ```
  ConstrainedBox expandedBox({double width, double height}) {
    return ConstrainedBox(
      constraints: BoxConstraints.expand(
        width: width,
        height: height,
      ),
      child: this,
    );
  }

  /// Allows combining widgets into a Column
  ///
  /// Use  + to combine any two widget into a column. If a Column is being added
  /// to the widget, then that widget is placed at the top of the Column.
  /// The Following will each result in one Column:
  /// ```dart
  /// Container() + Container();
  /// Container() +  Column(...);
  /// ```
  Column operator +(Widget combine) {
    if (combine is Column) {
      return combine.copyWith(children: [this, ...combine.children]);
    }
    return Column(
      children: <Widget>[this, combine],
    );
  }

  /// Allows combining widgets into a Row
  ///
  /// Use | to combine any two widget into a Row. If a Row is added to a widget,
  /// then that widget is placed at the in front of the Row.
  /// The Following will each result in one Row:
  /// ```dart
  /// Container() | Container();
  /// Container() | Row(...);
  /// ```
  Row operator |(Widget combine) {
    if (combine is Row) {
      return combine.copyWith(children: [this, ...combine.children]);
    }
    return Row(
      children: <Widget>[this, combine],
    );
  }

  /// Allows combining widgets into a Stack
  ///
  /// Use & to combine any two widget into a Stack. If a Stack is added to a
  /// widget, then that widget is placed at the bottom of the Stack.
  /// The Following will each result in one Stack:
  /// ```dart
  /// Container() & Container();
  /// Container() & Stack(...);
  /// ```
  Stack operator &(Widget combine) {
    if (combine is Stack) {
      return combine.copyWith(children: [this, ...combine.children]);
    }
    return Stack(
      children: <Widget>[this, combine],
    );
  }
}
