import 'package:flutter/widgets.dart';
import 'extendedAlignments.dart';

extension ExtendedWrap on Wrap {
  /// Returns a Wrap with any number of the properties changed.
  ///
  /// This is used for the library to change the Wrap properties. You can use
  /// this to change properties after creating a Wrap.
  /// Use it like so:
  /// ```dart
  /// Wrap(...).copyWith(direction: Axis.vertical);
  /// ```
  /// This can be used to create a Row and change properties afterwards.
  Wrap copyWith({
    List<Widget> children,
    double spacing,
    double runSpacing,
    WrapAlignment runAlignment,
    Axis direction,
    VerticalDirection verticalDirection,
    WrapCrossAlignment crossAxisAlignment,
    TextDirection textDirection,
    WrapAlignment alignment,
  }) {
    return Wrap(
      children: children ?? this.children,
      spacing: spacing ?? this.spacing,
      runSpacing: runSpacing ?? this.runSpacing,
      runAlignment: runAlignment ?? this.runAlignment,
      direction: direction ?? this.direction,
      verticalDirection: verticalDirection ?? this.verticalDirection,
      crossAxisAlignment: crossAxisAlignment ?? this.crossAxisAlignment,
      textDirection: textDirection ?? this.textDirection,
      alignment: alignment ?? this.alignment,
    );
  }

  /// Allows adding Widgets to a Wrap
  ///
  /// When a Widget is being added to a Wrap, if that Widget is a Wrap and the
  /// directional is horizontal, then its children are appended to the children
  /// of the first widget, and if that Widget is anything else, that widget is
  /// appended to the children of the Wrap.
  /// The Following will result in one Wrap with Direction horizontal:
  /// ```dart
  /// Wrap(...,direction:Axis.horizontal) | Wrap(...,direction:Axis.horizontal);
  /// Wrap(...,direction:Axis.horizontal) | Container();
  /// ```
  Wrap operator |(Widget combine) {
    if (combine is Wrap &&
        this.direction == Axis.horizontal &&
        combine.direction == Axis.horizontal) {
      return this.copyWith(children: [...this.children, ...combine.children]);
    } else {
      return this.copyWith(children: [...this.children, combine]);
    }
  }

  /// Allows adding Widgets to a Wrap
  ///
  /// When a Widget is being added to a Wrap, if that Widget is a Wrap and the
  /// directional is vertical, then its children are appended to the children
  /// of the first widget, and if that Widget is anything else, that widget is
  /// appended to the children of the Wrap.
  /// The Following will result in one Wrap with Direction vertical:
  /// ```dart
  /// Wrap(...,direction:Axis.vertical) + Wrap(...,direction:Axis.vertical);
  /// Wrap(...,direction:Axis.vertical) + Container();
  /// ```
  Wrap operator +(Widget combine) {
    if (combine is Wrap &&
        this.direction == Axis.vertical &&
        combine.direction == Axis.vertical) {
      return this.copyWith(children: [...this.children, ...combine.children]);
    } else {
      return this.copyWith(children: [...this.children, combine]);
    }
  }

  /// Turns a Wrap into a Row or Column based on direction
  ///
  /// Most of the properties of the Wrap will be preserved where possible.
  ///
  /// Use it like so:
  /// ```dart
  /// Wrap(...).unwrap();
  /// ```
  Flex unwrap({MainAxisSize mainAxisSize, TextBaseline textBaseline}) {
    if (this.direction == Axis.horizontal) {
      return Row(
        children: this.children,
        crossAxisAlignment: this.crossAxisAlignment.crossAxisAlignment(),
        verticalDirection: this.verticalDirection,
        textDirection: this.textDirection,
        mainAxisAlignment: this.alignment.mainAxisAlignment(),
        mainAxisSize: mainAxisSize,
        textBaseline: textBaseline,
      );
    } else {
      return Column(
        children: this.children,
        crossAxisAlignment: this.crossAxisAlignment.crossAxisAlignment(),
        verticalDirection: this.verticalDirection,
        textDirection: this.textDirection,
        mainAxisAlignment: this.alignment.mainAxisAlignment(),
        mainAxisSize: mainAxisSize,
        textBaseline: textBaseline,
      );
    }
  }
}
